@@ -38,12 +38,13 @@ import traceback as _traceback
 from mysql.connector import Error
 from colorama import Fore as _Fore
 from colorama import Style as _Style
-from colemen_config import _db_column_type,_db_table_type,_db_mysql_delete_query_type,_db_mysql_select_query_type,_db_mysql_update_query_type
+from colemen_config import _db_column_type,_db_table_type,_db_mysql_delete_query_type,_db_mysql_select_query_type,_db_mysql_update_query_type,_db_mysql_insert_query_type
 import colemen_utilities.dict_utils as _obj
 # import colemen_utilities.file_utils as _cfu
 # import colemen_utilities.directory_utils as _dirs
 import colemen_utilities.string_utils as _csu
 # import colemen_utilities.list_utils as _lu
+from colemen_utilities.database_utils.MySQL.InsertQuery import InsertQuery
 from colemen_utilities.database_utils.MySQL.SelectQuery import SelectQuery
 from colemen_utilities.database_utils.MySQL.UpdateQuery import UpdateQuery
 from colemen_utilities.database_utils.MySQL.DeleteQuery import DeleteQuery
@@ -103,8 +104,8 @@ class MySQLDatabase:
             `method_name`: MySQLDatabase
             * @xxx [12-13-2022 12:31:27]: documentation for MySQLDatabase
         '''
-        
-        
+
+
         self._dbm = None
         '''The Database manager instance.'''
 
@@ -397,8 +398,9 @@ class MySQLDatabase:
         # print(f"sql:{sql}")
         if self.connect() is False:
             return default
-        _log(f"sql:{sql}","cyan")
-        _log(f"args:{args}","cyan")
+        
+        # _log(f"sql:{sql}","cyan")
+        # _log(f"args:{args}","cyan")
 
         if self.run(sql,args):
             return self.fetchall()
@@ -406,7 +408,7 @@ class MySQLDatabase:
 
 
 
-    def insert_to_table(self,table_name:str,data:dict):
+    def insert_to_table(self,table_name:str,data:dict,cerberus_validate:bool=False):
         '''
             Insert a dictionary of data into the table specified
             ----------
@@ -431,8 +433,8 @@ class MySQLDatabase:
             `method_name`: insert_to_table
             * @xxx [12-13-2022 11:46:06]: documentation for insert_to_table
         '''
-        tb = self.get_table(table_name)
-        data = self.correlate_to_table(table_name,data,crud="create",cerberus_validate=True)
+        # tb = self.get_table(table_name)
+        data = self.correlate_to_table(table_name,data,crud="create",cerberus_validate=cerberus_validate)
         from colemen_utilities.sql_utils import insert_from_dict
         if len(data.keys()) == 0:
             _log("No keys in the data dict were correlated to columns in the table.","warning")
@@ -446,17 +448,33 @@ class MySQLDatabase:
         _log("Failed to execute SQL.","warning")
         return False
 
-    def select_query(self,table_name:str)->_db_mysql_select_query_type:
+    def insert_query(self,table:_Union[str,_db_table_type])->_db_mysql_insert_query_type:
+        '''Create a new insert query instance for the table provided.'''
+        if isinstance(table,(str)):
+            return InsertQuery(database=self,table_name=table)
+        else:
+            return InsertQuery(database=self,table=table)
+
+    def select_query(self,table:_Union[str,_db_table_type])->_db_mysql_select_query_type:
         '''Create a new select query instance'''
-        return SelectQuery(database=self,table_name=table_name)
+        if isinstance(table,(str)):
+            return SelectQuery(database=self,table_name=table)
+        else:
+            return SelectQuery(database=self,table=table)
 
-    def update_query(self,table_name:str)->_db_mysql_update_query_type:
+    def update_query(self,table:_Union[str,_db_table_type])->_db_mysql_update_query_type:
         '''Create a new update query instance'''
-        return UpdateQuery(database=self,table_name=table_name)
+        if isinstance(table,(str)):
+            return UpdateQuery(database=self,table_name=table)
+        else:
+            return UpdateQuery(database=self,table=table)    
 
-    def delete_query(self,table_name:str)->_db_mysql_delete_query_type:
+    def delete_query(self,table:_Union[str,_db_table_type])->_db_mysql_delete_query_type:
         '''Create a new delete query instance'''
-        return DeleteQuery(database=self,table_name=table_name)
+        if isinstance(table,(str)):
+            return DeleteQuery(database=self,table_name=table)
+        else:
+            return DeleteQuery(database=self,table=table)    
 
     # def get_from_table(self,table_name:str,data:dict,**kwargs):
     #     limit = _obj.get_kwarg(['limit'],self.get_limit,(int),**kwargs)
@@ -1094,96 +1112,200 @@ class MySQLDatabase:
         output = {}
         if crud_type in ['create']:
             if tb.has_timestamp_column:
-                output['timestamp'] = datetime.datetime.now(tz=datetime.timezone.utc).timestamp()
+                output['timestamp'] = int(datetime.datetime.now(tz=datetime.timezone.utc).timestamp())
 
             if tb.has_modified_timestamp_column:
-                output['modified_timestamp'] = datetime.datetime.now(tz=datetime.timezone.utc).timestamp()
+                output['modified_timestamp'] = int(datetime.datetime.now(tz=datetime.timezone.utc).timestamp())
 
             if tb.has_hash_id:
                 output['hash_id'] = tb.gen_hash_id()
 
         if crud_type in ['update']:
             if tb.has_modified_timestamp_column:
-                output['modified_timestamp'] = datetime.datetime.now(tz=datetime.timezone.utc).timestamp()
+                output['modified_timestamp'] = int(datetime.datetime.now(tz=datetime.timezone.utc).timestamp())
 
         if crud_type in ['delete']:
             if tb.has_modified_timestamp_column:
-                output['modified_timestamp'] = datetime.datetime.now(tz=datetime.timezone.utc).timestamp()
+                output['modified_timestamp'] = int(datetime.datetime.now(tz=datetime.timezone.utc).timestamp())
 
             if tb.has_deleted_column:
-                output['deleted'] = datetime.datetime.now(tz=datetime.timezone.utc).timestamp()
+                output['deleted'] = int(datetime.datetime.now(tz=datetime.timezone.utc).timestamp())
 
 
         return output
 
 
-
-
     def correlate_to_table(self,table_name:str,data:dict,**kwargs):
-        cerberus_validate = _obj.get_kwarg(['cerberus_validate'],False,(bool),**kwargs)
+        '''
+            Correlate a dictionary of data to the columns of the table.
+
+            ----------
+
+            Arguments
+            -------------------------
+            `table_name` {str}
+                The name of the table to compare the dictionary to.
+
+            `data` {dict}
+                The dictionary to compare.
+
+            Keyword Arguments
+            -------------------------
+            `arg_name` {type}
+                arg_description
+
+            Return {type}
+            ----------------------
+            return_description
+
+            Meta
+            ----------
+            `author`: Colemen Atwood
+            `created`: 12-13-2022 14:02:34
+            `memberOf`: MySQLDatabase
+            `version`: 1.0
+            `method_name`: correlate_to_table
+            * @TODO []: documentation for correlate_to_table
+        '''
+        # cerberus_validate = _obj.get_kwarg(['cerberus_validate'],False,(bool),**kwargs)
         crud_type = _obj.get_kwarg(['crud','crud_type'],None,(str),**kwargs)
         default_on_fail = _obj.get_kwarg(['default_on_failure'],False,(bool),**kwargs)
+        validate_types = _obj.get_kwarg(['validate_types'],True,(bool),**kwargs)
+        if isinstance(table_name,(str)):
+            tb = self.get_table(table_name)
+        else:
+            tb = table_name
 
-        tb = self.get_table(table_name)
         cols = tb.columns
         # cols = self.get_column_data(table_name)
         col:_db_column_type
-        output = self.__assign_default_meta_cols(tb,crud_type)
 
+        output = self.__assign_default_meta_cols(tb,crud_type)
 
         for col in cols:
             name = col.data.column_name
             if name in data:
                 value = data[name]
-                value_type = type(value).__name__
+                result = self.correlate_single_column(
+                    col,
+                    value,
+                    default_on_fail=default_on_fail
+                    )
+                if isinstance(result,(dict)):
+                    output = {**output,**result}
+                    continue
 
-                # if col.data.is_timestamp:
-                #     value = datetime.datetime.now(tz=datetime.timezone.utc).timestamp()
-                #     output[name] = value
-                #     continue
-                # if col.data.is_hash_id:
-                #     value = datetime.datetime.now(tz=datetime.timezone.utc).timestamp()
+                # value_type = type(value).__name__
+                # if validate_types is False:
                 #     output[name] = value
                 #     continue
 
-                if cerberus_validate:
-                    validation_schema = col.validation.cerberus_schema(crud_type)
-                    if validation_schema is None:
-                        a = "a"
-                        if crud_type == "update":
-                            a= "an"
-                        _log(f"{name} is not an allowed column in {a} {crud_type} operation","warning")
-                        continue
+                # # if cerberus_validate:
+                #     # validation_schema = col.validation.cerberus_schema(crud_type)
+                #     # if validation_schema is None:
+                #     #     a = "a"
+                #     #     if crud_type == "update":
+                #     #         a= "an"
+                #     #     _log(f"{name} is not an allowed column in {a} {crud_type} operation","warning")
+                #     #     continue
 
-                    from colemen_utilities.validate_utils.cerberus import single_value
-                    success,errors,output_data = single_value(name,validation_schema,value)
-                    if success is False:
-                        for k,v in errors.items():
-                            _log(f"Value for {k} failed validation: {v}","warning")
-                        continue
-                    else:
-                        _log(f"Successfully validated {name}","success")
-                        output[name] = output_data[name]
-                        # continue
+                #     # from colemen_utilities.validate_utils.cerberus import single_value
+                #     # success,errors,output_data = single_value(name,validation_schema,value)
+                #     # if success is False:
+                #     #     for k,v in errors.items():
+                #     #         _log(f"Value for {k} failed validation: {v}","warning")
+                #     #     continue
+                #     # else:
+                #     #     _log(f"Successfully validated {name}","success")
+                #     #     output[name] = output_data[name]
+                #     #     # continue
+
+                # # @Mstep [IF] if the value_type matches the column's supported data type (or its python equivalent)
+                # if value_type in col.data.py_data_type:
+                #     # @Mstep [] add the key value to the output
+                #     output[name] = value
+                # # @Mstep [ELSE] if the value_type does not match the column's supported data type
+                # else:
+
+                #     # @Mstep [IF] if the column contains a boolean value and the value is an int or string.
+                #     if col.validation.is_boolean is True and value_type in ['string','int']:
+                #         # @Mstep [] convert the value to a boolean.
+                #         from colemen_utilities.type_utils import to_bool
+                #         # @Mstep [] add the key value to the output
+                #         output[name] = to_bool(to_bool)
+                #         continue
+
+                #     # @Mstep [IF] if the value is None and the column is nullable
+                #     if value is None and col.data.is_nullable is True:
+                #         # @Mstep [] add the key value to the output
+                #         output[name] = value
+                #         continue
+
+                #     # @Mstep [IF] if the value is an int or float and the column only supports strings
+                #     if col.data.py_data_type in ['string'] and isinstance(value,(int,float)):
+                #         # @Mstep [] convert the value to a string
+                #         # @Mstep [] add the key value to the output
+                #         output[name] = str(value)
+                #         continue
 
 
-                if value_type in col.data.py_data_type:
-                    output[name] = value
-                else:
-                    if col.validation.is_boolean is True and value_type in ['string','int']:
-                        from colemen_utilities.type_utils import to_bool
-                        output[name] = to_bool(to_bool)
-                        continue
-                    if value is None and col.data.is_nullable is True:
-                        output[name] = value
-                    else:
-                        if default_on_fail is True:
-                            output[name] = col.data.default
-                        else:
-                            _log(f"Value for {name} expected: {col.validation.py_data_type} --- found: {value_type}","warning")
+                #     # else:
+                #     if default_on_fail is True:
+                #         output[name] = col.data.default
+                #     else:
+                #         _log(f"Value for {name} expected: {col.validation.py_data_type} --- found: {value_type}","warning")
 
         return output
 
+    def correlate_single_column(self,column:_db_column_type,value,**kwargs):
+        output = {}
+
+
+
+        default_on_fail = _obj.get_kwarg(["default_on_fail"],False,(bool),**kwargs)
+        name = column.data.column_name
+        value_type = type(value).__name__
+
+
+        # @Mstep [IF] if the value_type matches the column's supported data type (or its python equivalent)
+        if value_type in column.data.py_data_type:
+            # @Mstep [] add the key value to the output
+            output[name] = value
+        # @Mstep [ELSE] if the value_type does not match the column's supported data type
+        else:
+
+            # @Mstep [IF] if the column contains a boolean value and the value is an int or string.
+            if column.validation.is_boolean is True and value_type in ['string','int']:
+                # @Mstep [] convert the value to a boolean.
+                from colemen_utilities.type_utils import to_bool
+                # @Mstep [] add the key value to the output
+                output[name] = to_bool(to_bool)
+                return output
+
+            # @Mstep [IF] if the value is None and the column is nullable
+            if value is None and column.data.is_nullable is True:
+                # @Mstep [] add the key value to the output
+                output[name] = value
+                return output
+
+            # @Mstep [IF] if the value is an int or float and the column only supports strings
+            if column.data.py_data_type in ['string'] and isinstance(value,(int,float)):
+                # @Mstep [] convert the value to a string
+                # @Mstep [] add the key value to the output
+                output[name] = str(value)
+                return output
+
+
+            # else:
+            if default_on_fail is True:
+                output[name] = column.data.default
+            else:
+                _log(f"Value for {name} expected: {column.validation.py_data_type} --- found: {value_type}","warning")
+                output = False
+        return output
+
+
+
 
 def _to_statement_list(sql):
     sql = sql.replace(";", ";STATEMENT_END")
